import java.awt.BorderLayout;
import java.awt.List;
import java.io.*;
import java.net.URL;
import java.net.MalformedURLException;

/** 
 * This class opens a URL and shows upcoming birthdays in a window. 
 *
 * Run this applet using a browser and an HTML page containing the tag:
 * <applet code="DateCheckIII.class" width="600" height="400">
 *    <param name="dateURL" value="birthdays.txt">
 *    <param name="lookaheadDays" value="30">
 *    This browser cannot run Java.
 * </applet>
 *
 * @author  Dan Becker, beckerdo@us.ibm.com
 * @version 07/29/99     
 */
public class DateCheckIII extends java.applet.Applet {
   // Constants
   public static final boolean VERBOSE = true;
   public static final String [] PARAMETERNAMES = {
      "dateURL", "lookaheadDays"
   };

   // Constructors
   /** The "no arg" constructor initializes the class. 
     * It is provided automatically if you don't write it.
     * The super method calls this classes parent (Applet) constructor.
     */
   public DateCheckIII() {
      super();
      if ( VERBOSE ) System.out.println( "constructor" );
      // We should initialize fields to a know state here. Omitted.
   }

   // Applet role
   /** init is called when the applet is created. */
   public void init() {
      if ( VERBOSE ) System.out.println( "init" );

      // Get applet parameters.
      getParameters( PARAMETERNAMES );
   
      // Create window and layout to place date information.
      createGUI();

      // Parse URL stream for dates.
      parseDates( dateURL );
   }
   /** start is called when the applet page is visited, after init. */
   public void start( ) { 
      if ( VERBOSE ) System.out.println( "start" );
   } 
   /** stop is called when the applet page is left, before destory. */
   public void stop( ) {
      if ( VERBOSE ) System.out.println( "stop" );
   } 
   /** destroy is called when the applet is done and must be cleaned up. */
   public void destroy( ) {
      if ( VERBOSE ) System.out.println( "destroy" );
   } 
 
   // Methods
   /** Sets fields based on given parameter names. */
   public void getParameters( String [] parameters ) {
      for (int i = 0; i < parameters.length; i++ ) {
         String parameterName = parameters[ i ];
         String value = getParameter( parameterName );
         if ( VERBOSE ) System.out.println( "Parameter name=" + parameterName + ", value=" + value );
         if ( value == null )
            throw new IllegalArgumentException( "no parameter provided for " + parameterName );

         // Use the parameter name to set a field in this class.
         if ( parameterName.equals( "dateURL" ) ) {
            dateURL = value;
         } else if ( parameterName.equals( "lookaheadDays" ) ) {
            lookaheadDays = Integer.parseInt( value );
         } else {
            throw new IllegalArgumentException( "unknown parameter name " + parameterName );
         } /* endif */
      } /* endfor */
   } // getParameters

   /** Create the graphical user interface, windows and layout. */
   public void createGUI() {
      // Create a label
      setLayout( new BorderLayout() );
      dateList = new List();
      add( "Center", dateList );
   } // createWindows

   /** Opens the given URL as a stream and checks each line for nearby dates. */
   public void parseDates( String dateURL ) {
      try {
         // Open the location.
         URL location = new URL( getCodeBase(), dateURL );

         try {
            // Put a few readers together to interpret the location.
            InputStreamReader isr = new InputStreamReader( location.openStream() );
            LineNumberReader lnr = new LineNumberReader( isr );
   
            String line = lnr.readLine();
            while( line != null ) {
               if ( !line.startsWith( "//" ) ) {
                  StringTokenizer st = new StringTokenizer( line, "," );
      
                  // Parse date data from file.
                  String dateToken = st.nextToken();
                  String person = st.nextToken();
                  if ( VERBOSE ) System.out.println( "Checking date " + dateToken + ", person is " + person );
  
               } // if
               line = lnr.readLine();
            } // while
         } catch ( IOException e ) {
            System.out.println( "Problem reading from URL stream: " + e );
         } // catch
      } catch ( MalformedURLException e ) {
         System.out.println( "Problem with URL " + dateURL + ": " + e );
      } // catch
   } // parseDates

   // Fields
   protected String dateURL;
   protected int lookaheadDays;
   protected List dateList;
}
