/********************************************************************
* jedadvlib.js                                                         *
*                                                                   *
* Ray Wong                                                          *
* Last updated March 5, 1999.                                       *
*                                                                   *
* These functions allow dynamic HTML effects to be added to any     *
* web page and will work on both Netscape Communicator 4.0+ and MS  *
* Internet Explorer 4.0+ browsers.                                  *
*                                                                   *
********************************************************************/

var animRate = 25;    // Basically sets the update rate for animations.

function slideLayer(name, x, y, speed, code) 
{

  var layer = getLayer(name);

  // If the layer is currently being moved, cancel it.
 
  if (layer.slideID && layer.slideID != null)
    clearTimeout(layer.slideID);

  // Fix up all parameters.

  if (!code)
    code = "";

  // If any parameter other than 'name' is not an Array, make it one.

  x     = makeArray(x);
  y     = makeArray(y);
  speed = makeArray(speed);
  code  = makeArray(code);

  // Pad all arrays to the same length.

  var max = Math.max(x.length, Math.max(y.length, Math.max(speed.length, code.length)));
  while (x.length < max)

    x[x.length] = x[x.length - 1];
  while (y.length < max)
    y[y.length] = y[y.length - 1];
  while (speed.length < max)
    speed[speed.length] = speed[speed.length - 1];
  while (code.length < max)
    code[code.length] = code[code.length - 1];

  // Call the slide function with array parameters.

  doSlide(name, x, y, speed, code);
}

function doSlide(name, x, y, speed, code) 
{

  var layer = getLayer(name);

  var hrzn, vert, left, top, steps;

  if (ns) 
  {
    hrzn = x[0] - layer.left;
    vert = y[0] - layer.top;
    left = layer.left;

    top  = layer.top;
  }

  if (ie) 
  {
    hrzn = x[0] - layer.pixelLeft;
    vert = y[0] - layer.pixelTop;
    left = layer.pixelLeft;

    top  = layer.pixelTop;
  }

  // Calculate how many steps it will take and the size of each step.

  steps = (Math.max(Math.abs(hrzn), Math.abs(vert)) / speed[0]) * (1000 / animRate);

  if (steps <= 0)
    return;

  // Save remaining array values.

  layer.slideX     = x.slice(1, x.length);
  layer.slideY     = y.slice(1, y.length);
  layer.slideSpeed = speed.slice(1, speed.length);
  layer.slideCode  = code.slice(1, code.length);

  // Set up movement values.

  layer.slideLeft   = left;

  layer.slideTop    = top;

  layer.slideDx     = hrzn / steps;
  layer.slideDy     = vert / steps;
  layer.slideFinalX = x[0];

  layer.slideFinalY = y[0];

  layer.slideSteps  = Math.floor(steps);

  layer.slideAction = code[0];


  // Start the slide.

  slideStep(name);
}

function slideStep(name) 
{
  // Moves the layer one step.

  var layer = getLayer(name);

  layer.slideLeft += layer.slideDx;
  layer.slideTop  += layer.slideDy;

  if (ns)
    layer.moveTo(layer.slideLeft, layer.slideTop);
  if (ie) 
  {
    layer.left = layer.slideLeft;
    layer.top  = layer.slideTop;
  }

  // If more steps remain, call this function again.

  if (layer.slideSteps-- > 0)
    layer.slideID = setTimeout('slideStep("' + name + '")', animRate);
  else 
  {  
  	// Otherwise, move layer to final position, execute any code and look for more slide parameters.

   if (ns)
      layer.moveTo(layer.slideFinalX, layer.slideFinalY);
    if (ie) 
	{
      layer.left = layer.slideFinalX;
      layer.top  = layer.slideFinalY;
    }

    // Execute code, if any.

    layer.slideID = null;
    if (layer.slideAction != "")
      eval(layer.slideAction);

    // If any more slide parameters are left, start a new slide.

    if (layer.slideX.length > 0)
      doSlide(name, layer.slideX, layer.slideY, layer.slideSpeed, layer.slideCode);
    else
      layer.slideID = null;
  }
}

function clipLayer(name, clipleft, cliptop, clipright, clipbottom) 
{

  var layer = getLayer(name);

  if (ns) 
  {
    layer.clip.left   = clipleft;
    layer.clip.top    = cliptop;
    layer.clip.right  = clipright;
    layer.clip.bottom = clipbottom;
  }
  if (ie)
    layer.clip = 'rect(' + cliptop + ' ' +  clipright + ' ' + clipbottom + ' ' + clipleft +')';
}

function swipeLayer(name, clipleft, cliptop, clipright, clipbottom, speed, code) 
{

  var layer = getLayer(name);

  // If the layer is currently being clipped, cancel it.
 
  if (layer.swipeID && layer.swipeID != null)
    clearTimeout(layer.swipeID);

  // Fix up all parameters.

  if (!code)
    code = "";

  // If any parameter other than 'name' is not an Array, make it one.

  clipleft   = makeArray(clipleft);
  cliptop    = makeArray(cliptop);
  clipright  = makeArray(clipright);
  clipbottom = makeArray(clipbottom);
  speed      = makeArray(speed);
  code       = makeArray(code);

  // Pad all arrays to the same length.

  var max = Math.max(clipleft.length, Math.max(clipright.length, Math.max(cliptop.length, Math.max(clipbottom.length, Math.max(speed.length, code.length)))));
  while (clipleft.length < max)

    clipleft[clipleft.length] = clipleft[clipleft.length - 1];
  while (clipright.length < max)
    clipright[clipright.length] = clipright[clipright.length - 1];
  while (cliptop.length < max)
    cliptop[cliptop.length] = cliptop[cliptop.length - 1];
  while (clipbottom.length < max)
    clipbottom[clipbottom.length] = clipbottom[clipbottom.length - 1];
  while (speed.length < max)
    speed[speed.length] = speed[speed.length - 1];
  while (code.length < max)
    code[code.length] = code[code.length - 1];

  // Call the swipe function with array parameters.

  doSwipe(name, clipleft, cliptop, clipright, clipbottom, speed, code)
}

function doSwipe(name, clipleft, cliptop, clipright, clipbottom, speed, code) 
{

  var layer = getLayer(name);

  var hrzn1, vert1;
  var hrzn2, vert2;
  var max1, max2;
  var steps;

  if (ns) 
  {
    hrzn1 = clipleft[0] - layer.clip.left;
    vert1 = cliptop[0] - layer.clip.top;
    hrzn2 = clipright[0] - layer.clip.right;
    vert2 = clipbottom[0] - layer.clip.bottom;
  }
  if (ie) 
  {
    if (!layer.clip)
      clipLayer(name, 0, 0, layer.pixelWidth, layer.pixelHeight);
    var clip = getClipValues(layer.clip);

    hrzn1 = clipleft[0] - clip[3];
    vert1 = cliptop[0] - clip[0];
    hrzn2 = clipright[0] - clip[1];
    vert2 = clipbottom[0] - clip[2];
  }
  max1 = Math.max(Math.abs(hrzn1), Math.abs(vert1));
  max2 = Math.max(Math.abs(hrzn2), Math.abs(vert2));
  steps = (Math.max(max1, max2) / speed[0]) * (1000 / animRate);

  if (steps <= 0)
    return;

  // Save remaining array values.

  layer.swipeClipleft   = clipleft.slice(1, clipleft.length);
  layer.swipeCliptop    = cliptop.slice(1, cliptop.length);
  layer.swipeClipright  = clipright.slice(1, clipright.length);
  layer.swipeClipbottom = clipbottom.slice(1, clipbottom.length);
  layer.swipeSpeed      = speed.slice(1, speed.length);
  layer.swipeCode       = code.slice(1, code.length);

  // Set up clipping values.

  if (ns) 
  {
    layer.swipeLeft   = layer.clip.left;
    layer.swipeTop    = layer.clip.top;
    layer.swipeRight  = layer.clip.right;
    layer.swipeBottom = layer.clip.bottom;
  }
  if (ie) 
  {
    layer.swipeLeft   = parseInt(clip[3], 10);
    layer.swipeTop    = parseInt(clip[0], 10);
    layer.swipeRight  = parseInt(clip[1], 10);
    layer.swipeBottom = parseInt(clip[2], 10);
  }
  layer.swipeDx1         = hrzn1 / steps;
  layer.swipeDy1         = vert1 / steps;
  layer.swipeDx2         = hrzn2 / steps;
  layer.swipeDy2         = vert2 / steps;
  
  layer.swipeFinalLeft   = clipleft[0];
  layer.swipeFinalTop    = cliptop[0];
  layer.swipeFinalRight  = clipright[0];
  layer.swipeFinalBottom = clipbottom[0];
  layer.swipeSteps       = Math.floor(steps);
  layer.swipeAction      = code[0];


  // Start the swipe.

  swipeStep(name);
}

function swipeStep(name) 
{

  var layer = getLayer(name);


  // Adjust the layer's clipping area by one step.

  layer.swipeLeft   += layer.swipeDx1;

  layer.swipeTop    += layer.swipeDy1;

  layer.swipeRight  += layer.swipeDx2;

  layer.swipeBottom += layer.swipeDy2;


  if (ns) 
  {
    layer.clip.left   = layer.swipeLeft;

    layer.clip.top    = layer.swipeTop;

    layer.clip.right  = layer.swipeRight;

    layer.clip.bottom = layer.swipeBottom;

  }
  if (ie)
    layer.clip = 'rect(' + layer.swipeTop + ' ' + layer.swipeRight + ' ' + layer.swipeBottom + ' ' + layer.swipeLeft +')';

  // If more steps remain, call this function again.

  if (--layer.swipeSteps > 0)
    layer.swipeID = setTimeout('swipeStep("' + name + '")', animRate);

  // Otherwise, set layer's final clip area, execute any code and look for more swipe parameters.

  else 
  {
    if (ns) 
	{
      layer.clip.left   = Math.round(layer.swipeFinalLeft);
      layer.clip.top    = Math.round(layer.swipeFinalTop);
      layer.clip.right  = Math.round(layer.swipeFinalRight);
      layer.clip.bottom = Math.round(layer.swipeFinalBottom);
    }
    if (ie)
      layer.clip = 'rect(' + layer.swipeFinalTop + ' ' + layer.swipeFinalRight + ' ' + layer.swipeFinalBottom + ' ' + layer.swipeFinalLeft +')';

    // Execute code, if any.

    if (layer.swipeAction != "")
      eval(layer.swipeAction);

    // If any more swipe parameters are left, start a new swipe.

    if (layer.swipeClipleft.length > 0)
      doSwipe(name, layer.swipeClipleft, layer.swipeCliptop, layer.swipeClipright, layer.swipeClipbottom, layer.swipeSpeed, layer.swipeCode);
    else
      layer.swipeID = null;
  }
}

function scrollLayer(name, dx, dy) 
{

  var cl = getClipLeft(name);
  var ct = getClipTop(name);
  var cr = getClipRight(name);
  var cb = getClipBottom(name);
  var l  = getLeft(name);
  var t  = getTop(name);

  // If scrolling the given amounts would move past the edges of the layer,
  // adjust the values so we stop right at the edge.

  if (cl + dx < 0)

    dx = -cl;

  else if (cr + dx > getWidth(name))
    dx = getWidth(name) - cr;
  if (ct + dy < 0)

    dy = -ct;

  else if (cb + dy > getHeight(name))
    dy = getHeight(name) - cb;

  // Move both the clipping region and the layer so that the contents move
  // but the viewable region of the layer appears fixed relative to the page.

  clipLayer(name, cl + dx, ct + dy, cr + dx, cb + dy);
  moveLayer(name, l - dx, t - dy);
}

function orbitLayer(name, x, y, a, b, start, num, rpm, rotation, code) 
{

  var layer = getLayer(name);
  var end, steps, delta;
  var x1, y1, x2, y2;

  // If the layer is currently in an orbit, cancel it.
 
  if (layer.orbitID && layer.orbitID != null)
    clearTimeout(layer.orbitID);

  // Convert starting angle to radians and find ending angle.

  start %= 360;
  start *= Math.PI / 180;
  end = start + num * 2 * Math.PI;

  // Convert rotation angle to radians.

  rotation %= 360;
  rotation *= Math.PI / 180;

  // Calculate how many steps it will take and the change in angle for each step.

  steps = 60000 * Math.abs(num) / (rpm * animRate);
  delta = (end - start) / steps;

  if (steps <= 0)
    return;

  // Set up movement values.

  layer.orbitX        = x;

  layer.orbitY        = y;

  layer.orbitA        = a;

  layer.orbitB        = b;

  layer.orbitAngle    = start;

  layer.orbitRotation = rotation;

  layer.orbitFinal    = end;

  layer.orbitDelta    = delta;

  layer.orbitSteps    = Math.floor(steps);

  layer.orbitAction   = code;


  // Move layer to starting position.

  x1 = a * Math.sin(start);
  y1 = b * Math.cos(start);
  x2 = x + Math.round(x1 * Math.cos(rotation) + y1 * Math.sin(rotation));
  y2 = y - Math.round(y1 * Math.cos(rotation) - x1 * Math.sin(rotation));
  moveLayer(name, x2, y2);

  // Start the orbit.

  orbitStep(name);
}

function orbitStep(name) 
{

  // Moves the layer one step.

  var layer = getLayer(name);

  var x1, y1, x2, y2;

  layer.orbitAngle += layer.orbitDelta;

  if (layer.orbitAngle > 2 * Math.PI)
    layer.orbitAngle -= 2 * Math.PI;
  else if (layer.orbitAngle < 0)

    layer.orbitAngle += 2 * Math.PI;
  x1 = layer.orbitA * Math.sin(layer.orbitAngle);
  y1 = layer.orbitB * Math.cos(layer.orbitAngle);
  x2 = layer.orbitX + Math.round(x1 * Math.cos(layer.orbitRotation) + y1 * Math.sin(layer.orbitRotation));
  y2 = layer.orbitY - Math.round(y1 * Math.cos(layer.orbitRotation) - x1 * Math.sin(layer.orbitRotation));
  if (ns)
    layer.moveTo(x2, y2);
  if (ie) 
  {
    layer.left = x2;

    layer.top  = y2;

  }

  // If more steps remain, call this function again.

  if (layer.orbitSteps-- > 0)
    layer.orbitID = setTimeout('orbitStep("' + name + '")', animRate);

  // Otherwise, move layer to final position and execute any code.

  else 
  {
    x1 = layer.orbitA * Math.sin(layer.orbitFinal);
    y1 = layer.orbitB * Math.cos(layer.orbitFinal);
    x2 = layer.orbitX + Math.round(x1 * Math.cos(layer.orbitRotation) + y1 * Math.sin(layer.orbitRotation));
    y2 = layer.orbitY - Math.round(y1 * Math.cos(layer.orbitRotation) - x1 * Math.sin(layer.orbitRotation));
    if (ns)
      layer.moveTo(x2, y2);
    if (ie) {
      layer.left = x2;
      layer.top  = y2;
    }

    // Execute code, if any.

    layer.orbitID = null;
    if (layer.orbitAction != "")
      eval(layer.orbitAction);

    // Clear the orbit ID.

    layer.orbitID = null;
  }
}

function orbitStartX(x, y, a, b, start, rotation) 
{

  var x1, y1, x2, y2;

  // Find the starting x-coordinate for the given orbit parameters.

  start %= 360;
  start *= Math.PI / 180;
  rotation %= 360;
  rotation *= Math.PI / 180;
  x1 = a * Math.sin(start);
  y1 = b * Math.cos(start);
  x2 = x + Math.round(x1 * Math.cos(rotation) + y1 * Math.sin(rotation));
  y2 = y - Math.round(y1 * Math.cos(rotation) - x1 * Math.sin(rotation));
  return Math.round(x2);
}

function orbitStartY(x, y, a, b, start, rotation) 
{

  var x1, y1, x2, y2;

  // Find the starting y-coordinate for the given orbit parameters.

  start %= 360;
  start *= Math.PI / 180;
  rotation %= 360;
  rotation *= Math.PI / 180;
  x1 = a * Math.sin(start);
  y1 = b * Math.cos(start);
  x2 = x + Math.round(x1 * Math.cos(rotation) + y1 * Math.sin(rotation));
  y2 = y - Math.round(y1 * Math.cos(rotation) - x1 * Math.sin(rotation));
  return y2;
}
function getClipLeft(name) 
{

  var layer = getLayer(name);

  if (ns)
    return(layer.clip.left);
	
  if (ie) 
  {
    var str =  layer.clip;
    if (!str)
      return(0);
    var clip = getClipValues(layer.clip);
    return(clip[3]);
  }
  
  return(null);
}

function getClipTop(name) 
{

  var layer = getLayer(name);

  if (ns)
    return(layer.clip.top);
	
  if (ie) 
  {
    var str =  layer.clip;
    if (!str)
      return(0);
	  
    var clip = getClipValues(layer.clip);
    return(clip[0]);
  }
  
  return(null);
}

function getClipRight(name) 
{

  var layer = getLayer(name);

  if (ns)
    return(layer.clip.right);
	
  if (ie) 
  {
    var str =  layer.clip;
    if (!str)
      return(layer.pixelWidth);
    var clip = getClipValues(layer.clip);
    return(clip[1]);
  }

  return(null);
}

function getClipBottom(name) 
{

  var layer = getLayer(name);

  if (ns)
    return(layer.clip.bottom);
	
  if (ie) {
    var str =  layer.clip;
    if (!str)
      return(layer.pixelHeight);
    var clip = getClipValues(layer.clip);
    return(clip[2]);
  }
  
  return(null);
}

function getClipWidth(name) 
{

  var layer = getLayer(name);

  if (ns)
    return(layer.clip.width);
	
  if (ie) {
    var str =  layer.clip;
    if (!str)
      return(layer.pixelWidth);
    var clip = getClipValues(layer.clip);
    return(clip[1] - clip[3]);
  }
    
  return(null);
}

function getClipHeight(name) 
{

  var layer = getLayer(name);

  if (ns)
    return(layer.clip.height);
  
  if (ie) 
  {
    var str =  layer.clip;
    if (!str)
      return(layer.pixelHeight);
    var clip = getClipValues(layer.clip);
    return(clip[2] - clip[0]);
  }
    
  return(null);
}

function getClipValues(str) 
{
  var clip = new Array();
  var i;

  // Parse out the clipping values for IE layers.

  i = str.indexOf("(");

  clip[0] = parseInt(str.substring(i + 1, str.length), 10);
  i = str.indexOf(" ", i + 1);
  clip[1] = parseInt(str.substring(i + 1, str.length), 10);
  i = str.indexOf(" ", i + 1);
  clip[2] = parseInt(str.substring(i + 1, str.length), 10);
  i = str.indexOf(" ", i + 1);
  clip[3] = parseInt(str.substring(i + 1, str.length), 10);
  return(clip);
}
